import Vault from "./vault";
/**
 * Credential object that aims to provide the password and salt used for
 * encryption and decryption of SecureVault data.
 *
 * @typedef {Object} EncCredential
 * @property {string} password - The encrypted password
 * @property {string} salt - The salt used for encryption
 */
type EncCredential = {
    password: string;
    salt: string;
};
/**
 * Function that returns a Promise which resolves to `EncCredential` object.
 *
 * @typedef {function} FuncEncCredential
 * @param {string} key - The key used for encryption
 * @returns {Promise<EncCredential>} A Promise that resolves to an Encrypted Credential
 */
type FuncEncCredential = (key: string) => Promise<EncCredential>;
/**
 * Configuration used for encryption and decryption of SecureVault data. If may
 * be an `EncCredential` object or a function that
 * returns a Promise which resolves to an `EncCredential` object.
 *
 * @typedef {Object} EncConfig
 * @property {EncCredential | FuncEncCredential | null} - The encrypted
 * configuration can be an Encrypted Credential, a function that returns a
 * Promise which resolves to an Encrypted Credential, or null. When the value
 * is null, the SecureVault will not encrypt or decrypt the data for the
 * specified key.
 */
type EncConfig = EncCredential | FuncEncCredential | null;
/**
 * The SecureVault class provides a secure way to store and retrieve data from
 * the Vault. The SecureVault class extends the Vault class and provides
 * encryption and decryption of data using the Web Cryptography API.
 */
declare class SecuredVault extends Vault {
    private encConfig;
    private keyCache;
    /**
     * Constructs a new instance of the SecuredVault class.
     * @constructor
     * @param {string} dbName - The name of the database.
     * @param {EncConfig} encConfig - The encrypted configuration.
     */
    constructor(dbName: string, encConfig: EncConfig);
    /**
     * Asynchronously sets the value of the specified key in the vault.
     * If the encryption configuration is not null and the value is not null or
     * undefined, the value is encrypted before being stored.
     *
     * @async
     * @param {string} key - The key of the item to set.
     * @param {any} value - The value of the item to set.
     * @param {any} meta - The metadata for the item (e.g., ttl, expiration) or
     *                     any other data that should be stored alongside the value.
     * @returns {Promise<void>} A Promise that resolves when the value has been set.
     */
    setItem(key: string, value: any, meta?: any): Promise<void>;
    /**
     * Asynchronously gets the value of the specified key from the vault.
     * If the encryption configuration is not null and the retrieved value is not
     * null, the value is decrypted before being returned.
     *
     * @async
     * @param {string} key - The key of the item to get.
     * @returns {Promise<any>} A Promise that resolves to the value of the specified key. If the key does not exist, the Promise resolves to null.
     */
    getItem(key: string): Promise<any>;
    private getKey;
}
export default SecuredVault;
