/**
 * Vault is an asynchronous key/value store similar to localStorage, but
 * provides a more flexible and powerful storage mechanism.
 */
export default class Vault {
    protected storageName: string;
    protected db: IDBDatabase | null;
    [key: string]: any;
    /**
     * Creates new custom instance of custom Vault Storage.
     * @param {string} [storageName] - The name of the storage.
     * @param {boolean} [isParent=false] - A flag to indicate if this instance
     * is a parent. This property should be ignored by the user unless they are
     * extending the Vault class.
     */
    constructor(storageName?: string, isParent?: boolean);
    /**
     * Set an item in the database with additional metadata.
     * @param {string} key - The key of the item.
     * @param {any} value - The value of the item.
     * @param {any} meta - The metadata for the item (e.g., ttl, expiration) or
     *                     any other data that should be stored alongside the value.
     * @returns {Promise<void>}
     */
    setItem(key: string, value: any, meta?: Record<string, any> | null): Promise<void>;
    /**
     * Get an item from the database.
     * @param {string} key - The key of the item.
     * @returns {Promise<any>} - The value of the item.
     */
    getItem(key: string): Promise<any>;
    /**
     * Remove an item from the database.
     * @param {string} key - The key of the item.
     * @returns {Promise<void>}
     */
    removeItem(key: string): Promise<void>;
    /**
     * Clear the database.
     * @returns {Promise<void>}
     */
    clear(): Promise<void>;
    /**
     * Get all keys in the database.
     * @returns {Promise<string[]>} - An array of keys.
     */
    keys(): Promise<string[]>;
    /**
     * Get the number of items in the database.
     * @returns {Promise<number>} - The number of items.
     */
    length(): Promise<number>;
    /**
     * Get an item's metadata from the database.
     * @param {string} key - The key of the item.
     * @returns {Promise<any>} - The metadata of the item.
     */
    getItemMeta(key: string): Promise<any>;
    protected init(): Promise<void>;
    protected do(operationType: IDBTransactionMode, operation: (store: IDBObjectStore) => IDBRequest): Promise<any>;
}
